package com.alterdekim.server.durak.component;

import com.alterdekim.server.durak.component.durak.ConnectionProcessor;
import com.alterdekim.server.durak.component.durak.ConnectionState;
import com.alterdekim.server.durak.component.durak.HandshakeProcessor;
import lombok.Getter;
import lombok.Setter;
import lombok.extern.slf4j.Slf4j;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.Socket;
import java.util.HashMap;
import java.util.Map;

@Slf4j
public class GameListener extends Thread {

    private Socket sock;
    private GameServer parent;

    @Getter
    @Setter
    private Integer uid;

    private InputStream in;
    private OutputStream out;

    private ConnectionState connectionState;

    private final Map<ConnectionState, ConnectionProcessor> connectionProcessors = new HashMap<>();

    public GameListener(Socket sock, GameServer parent, Integer uid) {
        try {
            this.sock = sock;
            this.parent = parent;
            this.uid = uid;
            this.in = sock.getInputStream();
            this.out = sock.getOutputStream();
            this.connectionState = ConnectionState.HANDSHAKE;
            this.connectionProcessors.put(ConnectionState.HANDSHAKE, new HandshakeProcessor(this.in, this.out, this.sock));
        } catch (IOException e) {
            log.error("GameListener constructor error", e);
        }
    }

    @Override
    public void run() {
        try {
            while(!sock.isClosed()) {
                this.connectionState = this.connectionProcessors.get(this.connectionState).process();
            }
            this.parent.close(this.getUid());
        } catch( Exception e ) {
            log.error("RTMPListener.run() error", e);
        } finally {
            try {
                sock.close();
            } catch (Exception e) {
                log.error("RTMPListener.run() error", e);
            }
        }
    }
}
