package com.alterdekim.server.durak.controller;

import com.alterdekim.server.durak.storage.StorageFileNotFoundException;
import com.alterdekim.server.durak.storage.StorageProperties;
import com.alterdekim.server.durak.storage.StorageService;
import jakarta.servlet.http.HttpServletRequest;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.context.properties.EnableConfigurationProperties;
import org.springframework.core.io.Resource;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

import java.nio.file.Files;
import java.nio.file.Path;

@Controller
@EnableConfigurationProperties(StorageProperties.class)
@Slf4j
public class FileServerController {

    private final StorageService storageService;

    @Autowired
    public FileServerController(StorageService storageService) {
        this.storageService = storageService;
    }

    @RequestMapping(value = "/durak/{*resPath}", method = RequestMethod.GET)
    @ResponseBody
    public ResponseEntity<Resource> serveFile(@PathVariable("resPath") String resPath, HttpServletRequest request) {
        try {
            String filename = request.getRequestURL().substring(request.getRequestURL().indexOf("/durak/"));
            Resource file = storageService.loadAsResource(filename);
            return ResponseEntity.ok().contentType(MediaType.parseMediaType("image/png")).body(file);
        } catch (Exception e) {
            log.error(e.getMessage());
        }
        return ResponseEntity.notFound().build();
    }

    @ExceptionHandler(StorageFileNotFoundException.class)
    public ResponseEntity<?> handleStorageFileNotFound(StorageFileNotFoundException exc) {
        return ResponseEntity.notFound().build();
    }
}
